@@ -1,15 +1,15 @@
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-from twisted.scripts.mktap import _tapHelper
+from twisted.application.service import ServiceMaker
 
-TwistedSSH = _tapHelper(
+TwistedSSH = ServiceMaker(
     "Twisted Conch Server",
     "twisted.conch.tap",
     "A Conch SSH service.",
     "conch")
 
-TwistedManhole = _tapHelper(
+TwistedManhole = ServiceMaker(
     "Twisted Manhole (new)",
     "twisted.conch.manhole_tap",
     ("An interactive remote debugger service accessible via telnet "
